<?php

namespace Opencart\Admin\Controller\Extension\Axepta\Payment;

use Axepta\Payment\Util;
use Opencart\System\Engine\Controller;

/**
 * Class Axepta
 */
class Axepta extends Controller
{

	/**
	 * @return void
	 */
	public function index()
	{
		$this->load->language('extension/axepta/payment/axepta');
		$this->document->setTitle($this->language->get('heading_title'));
		$this->document->addScript('../extension/axepta/admin/view/javascript/select2/js/select2.min.js');
		$this->document->addStyle('../extension/axepta/admin/view/javascript/select2/css/select2.css');
		$this->load->model('setting/setting');

		include_once(DIR_EXTENSION . 'axepta/system/library/payment-core/vendor/autoload.php');

		$data = [

			'payment_axepta_status' => $this->config->get('payment_axepta_status'),

			'payment_axepta_sandbox' => $this->config->get('payment_axepta_sandbox'),

			'payment_axepta_merchant_id' => $this->config->get('payment_axepta_merchant_id'),

			'payment_axepta_authorization_token' => $this->config->get('payment_axepta_authorization_token'),

			'payment_axepta_service_id' => $this->config->get('payment_axepta_service_id'),

			'payment_axepta_service_key' => $this->config->get('payment_axepta_service_key'),

			'payment_axepta_sort_order' => $this->config->get('payment_axepta_sort_order'),

			'payment_axepta_send_cart' => $this->config->get('payment_axepta_send_cart'),

			'payment_axepta_order_history' => $this->config->get('payment_axepta_order_history'),

			'payment_axepta_payment_title_default' => $this->language->get('text_axepta_payment_title_default'),
			'payment_axepta_payment_title'         => $this->config->get('payment_axepta_payment_title'),

			'payment_axepta_all_currencies' => Util::getSupportedCurrencies(),
			'payment_axepta_currencies'     => $this->config->get('payment_axepta_currencies')
				?: [],

			'footer'      => $this->load->controller('common/footer'),
			'header'      => $this->load->controller('common/header'),
			'column_left' => $this->load->controller('common/column_left'),

			'breadcrumbs' => [
				[
					'text' => $this->language->get('text_home'),
					'href' => $this->url->link('common/dashboard', 'user_token=' . $this->session->data['user_token']),
				],

				[
					'text' => $this->language->get('text_extension'),
					'href' => $this->url->link('marketplace/extension', 'user_token=' . $this->session->data['user_token'] . '&type=payment'),
				],

				[
					'text' => $this->language->get('heading_title'),
					'href' => $this->url->link('extension/axepta/payment/axepta', 'user_token=' . $this->session->data['user_token'], 'SSL'),
				],
			],

			'save' => $this->url->link('extension/axepta/payment/axepta|save', 'user_token=' . $this->session->data['user_token']),
			'back' => $this->url->link('marketplace/extension', 'user_token=' . $this->session->data['user_token'] . '&type=payment'),

		];

		$this->response->setOutput($this->load->view('extension/axepta/payment/axepta', $data));

		$current_axepta_settings = $this->model_setting_setting->getSetting('payment_axepta');

		if(isset($current_axepta_settings['axepta_error_tmp'])) {
			unset($current_axepta_settings['axepta_error_tmp']);
			$this->model_setting_setting->editSetting('axepta', $current_axepta_settings);
		}

		if(isset($current_axepta_settings['axepta_success_tmp'])) {
			unset($current_axepta_settings['axepta_success_tmp']);
			$this->model_setting_setting->editSetting('axepta', $current_axepta_settings);
		}
	}

	/**
	 * @return void
	 */
	public function save()
	{

		$this->load->language('extension/axepta/payment/axepta');
		$json = [];

		if(!$this->user->hasPermission('modify', 'extension/axepta/payment/axepta')) {
			$json['error']['warning'] = $this->language->get('error_permission');
		}
		if(!$json) {
			$this->load->model('setting/setting');
			$this->model_setting_setting->editSetting('payment_axepta', $this->request->post);
			$json['success'] = $this->language->get('text_success');
		}
		$this->response->addHeader('Content-Type: application/json');
		$this->response->setOutput(json_encode($json));
	}
}
