<?php

namespace Opencart\Catalog\Controller\Extension\Axepta\Payment;

use Axepta\Payment\Api;
use Axepta\Payment\Notification;
use Axepta\Payment\Util;
use Exception;
use Opencart\System\Engine\Controller;
use Opencart\System\Library\Log;

/**
 * Class Axepta
 */
class Axepta extends Controller
{

	/**
	 * plugin version
	 */
	const VERSION = '1.0.0';

	/**
	 * @return mixed
	 */
	public function index()
	{

		$this->load->language('extension/axepta/payment/axepta');

		$data['language'] = $this->config->get('config_language');

		return $this->load->view('extension/axepta/payment/axepta', $data);
	}

	/**
	 * @return void
	 */
	public function confirm()
	{

		include_once(DIR_EXTENSION . 'axepta/system/library/payment-core/vendor/autoload.php');

		$logger = new Log('axepta.log');

		$this->language->load('extension/axepta/payment/axepta');

		$json = [];

		try {

			if(!isset($this->session->data['order_id']) ||
			is_array($this->session->data['payment_method'])
				? $this->session->data['payment_method']['code'] !== 'axepta.axepta'
				: $this->session->data['payment_method'] !== 'axepta') {
				throw new Exception($this->language->get('error_payment_method'));
			}
		} catch(Exception $e) {
			$json['error'] = $e->getMessage();
		}

		if(!$json) {

			$this->load->model('checkout/order');

			$order_info = $this->model_checkout_order->getOrder($this->session->data['order_id']);

			$api = new Api(
				$this->config->get('payment_axepta_authorization_token'),
				$this->config->get('payment_axepta_merchant_id'),
				$this->config->get('payment_axepta_service_id'),
				($this->config->get('payment_axepta_sandbox')
					? Util::ENVIRONMENT_SANDBOX
					: Util::ENVIRONMENT_PRODUCTION),
			);

			$transaction = $api->createPaymentLink($api->prepareDataPaymentLink(
				$order_info['total'] * $order_info['currency_value'],
				strtoupper($order_info['currency_code']),
				$order_info['order_id'],
				'',
				$this->url->link('checkout/success'),
				$this->url->link('checkout/failure'),
				$order_info['firstname'],
				$order_info['lastname'],
				$order_info['email'],
				$this->url->link('extension/axepta/payment/axepta' . urlencode('|') . 'notification', '', true),
				$order_info['telephone']
			));

			if($transaction['success']
				&& (
					isset($transaction['body']['data']['paymentLink']['url'])
					&& $transaction['body']['data']['paymentLink']['url']
				)) {

				$this->model_checkout_order->editTransactionId($this->session->data['order_id'], $order_info['order_id']);
				$this->model_checkout_order->addHistory($this->session->data['order_id'], 1, 'Axepta payment link ID: ' . $transaction['body']['data']['paymentLink']['paymentId']);
				$json['redirect'] = $transaction['body']['data']['paymentLink']['url'];
			} else {
				$logger->write('Axepta problem while initialize transaction, transaction: ' . json_encode($transaction));

				$json['error'] = $this->language->get('text_axepta_payment_error');
			}
		}

		$this->response->addHeader('Content-Type: application/json');
		$this->response->setOutput(json_encode($json));
	}

	/**
	 * Support incoming notification (and validate)
	 *
	 * @throws Exception
	 */
	public function notification()
	{

		include_once(DIR_EXTENSION . 'axepta/system/library/payment-core/vendor/autoload.php');

		$notification = new Notification(
			$this->config->get('payment_axepta_service_id'),
			$this->config->get('payment_axepta_service_key')
		);

		$result_check_request_notification = $notification->checkRequest();
		$debug_mode = true;

		if(is_int($result_check_request_notification)) {
			echo Notification::formatResponse(
				Notification::NS_ERROR,
				$result_check_request_notification,
				$debug_mode
			);
			exit();
		}

		$this->load->model('checkout/order');

		$order = $this->model_checkout_order->getOrder($result_check_request_notification['payment']['orderId']);

		if(!$order) {
			echo Notification::formatResponse(
				Notification::NS_ERROR,
				Notification::NC_ORDER_NOT_FOUND,
				$debug_mode
			);
			exit();
		}

		if(!Notification::checkRequestAmount(
			$result_check_request_notification,
			Util::convertAmountToFractional($order['total'] * $order['currency_value']),
			strtoupper($order['currency_code']))
		) {

			echo Notification::formatResponse(
				Notification::NS_ERROR,
				Notification::NC_AMOUNT_NOT_MATCH,
				$debug_mode
			);
			exit();
		}

		$orderStatusBefore = $order['order_status_id'];
		$orderStatusAfter = null;

		if($order['order_status_id'] === '1' || $order['order_status_id'] === '10') {

			switch($result_check_request_notification['payment']['status']) {
				case Notification::PS_SETTLED:
					$orderStatusAfter = 02;
					$this->model_checkout_order->editTransactionId($order['order_id'], $result_check_request_notification['payment']['id']);
					$this->model_checkout_order->addHistory($order['order_id'], $orderStatusAfter);
					break;
				case Notification::PS_REJECTED:
					$orderStatusAfter = 10;
					$this->model_checkout_order->addHistory($order['order_id'], $orderStatusAfter);
					break;
				default:
					echo Notification::formatResponse(
						Notification::NS_OK,
						Notification::NC_ORDER_STATUS_NOT_CHANGED,
						$debug_mode,
						$orderStatusBefore
					);
					exit();
			}
		}

		echo Notification::formatResponse(
			Notification::NS_OK,
			Notification::NC_OK,
			$debug_mode,
			$orderStatusBefore,
			$orderStatusAfter
		);
	}

	/**
	 * @param array $order
	 *
	 * @return array
	 */
	private function getAddressDelivery($order)
	{

		// check if shipping data is provided - 'shipping_city' is enough
		if($order['shipping_city']) {
			return [
				'shipping_city'       => $order['shipping_city'],
				'shipping_name'       => $order['shipping_firstname'] . ' ' . $order['shipping_lastname'],
				'telephone'           => $order['telephone'],
				'shipping_address'    => isset($order['shipping_address_2'])
					? $order['shipping_address_1'] . ' ' . $order['shipping_address_2']
					: $order['shipping_address_1'],
				'shipping_iso_code_2' => $order['shipping_iso_code_2'],
				'shipping_postcode'   => $order['shipping_postcode'],
			];
		} else {
			return [
				'shipping_city'       => $order['payment_city'],
				'shipping_name'       => $order['payment_firstname'] . ' ' . $order['payment_lastname'],
				'telephone'           => $order['telephone'],
				'shipping_address'    => isset($order['payment_address_2'])
					? $order['payment_address_1'] . ' ' . $order['payment_address_2']
					: $order['payment_address_1'],
				'shipping_iso_code_2' => $order['payment_iso_code_2'],
				'shipping_postcode'   => $order['payment_postcode'],
			];
		}
	}
}
